@@ -26,26 +26,32 @@ def find_caller(stack):
         if module.__name__.startswith('sqlalchemy'):
             continue
         return (module.__name__,) + tuple(frame[2:4]) + (frame[4][0].strip(),)
-    log.warning('session started/ended for unknown origin')
+    log.warning('Transaction from unknown origin')
     return None, None, None, None
 
 
 def after_begin(session, transaction, connection):
     caller_info = find_caller(inspect.stack()[1:])
     with open_transactions_lock:
-        if open_transactions:
-            log.warning('session 0x%08X %s used with already open one(s) %s',
+        if any(info[1] is not connection.connection for info in open_transactions.itervalues()):
+            log.warning('Sessions from 2 threads! Transaction 0x%08X opened %s Already open one(s): %s',
                         id(transaction), caller_info, open_transactions)
+        elif open_transactions:
+            log.debug('Transaction 0x%08X opened %s Already open one(s): %s',
+                      id(transaction), caller_info, open_transactions)
         else:
-            log.debug('session connection 0x%08X opened %s', id(transaction), caller_info)
+            log.debug('Transaction 0x%08X opened %s', id(transaction), caller_info)
         # Store information about this transaction
-        open_transactions[transaction] = (time.time(),) + caller_info
+        open_transactions[transaction] = (time.time(), connection.connection) + caller_info
 
 
-def before_flush(session, flush_context, instances):
+def after_flush(session, flush_context):
     if session.new or session.deleted or session.dirty:
-        log.debug('session connection 0x%08X writing: %s %s %s',
-                  id(session.transaction), session.new, session.deleted, session.dirty)
+        caller_info = find_caller(inspect.stack()[1:])
+        with open_transactions_lock:
+            tid = next(id(t) for t in session.transaction._iterate_parents() if t in open_transactions)
+            log.debug('Transaction 0x%08X writing %s new: %s deleted: %s dirty: %s',
+                      tid, caller_info, tuple(session.new), tuple(session.deleted), tuple(session.dirty))
 
 
 def after_end(session, transaction):
@@ -55,8 +61,8 @@ def after_end(session, transaction):
             # Transaction was created but a connection was never opened for it
             return
         open_time = time.time() - open_transactions[transaction][0]
-        msg = 'session connection 0x%08X closed %s (open time %s)' % (id(transaction), caller_info, open_time)
-        log.warning(msg) if open_time > 5 else log.debug(msg)
+        msg = 'Transaction 0x%08X closed %s (open time %s)' % (id(transaction), caller_info, open_time)
+        log.warning(msg) if open_time > 2 else log.debug(msg)
         del open_transactions[transaction]
 
 
@@ -64,7 +70,7 @@ def after_end(session, transaction):
 def debug_warnings(manager):
     if manager.options.debug_db_sessions:
         sqlalchemy.event.listen(Session, 'after_begin', after_begin)
-        sqlalchemy.event.listen(Session, 'before_flush', before_flush)
+        sqlalchemy.event.listen(Session, 'after_flush', after_flush)
         sqlalchemy.event.listen(Session, 'after_transaction_end', after_end)
 
 
