@@ -69,6 +69,9 @@ class TaskQueue(object):
         log.debug('task queue shutdown requested')
         if finish_queue:
             self._shutdown_when_finished = True
+            if self.run_queue.qsize():
+                log.info('There are %s tasks still executing. Shutdown will commence when they have completed.' %
+                         self.run_queue.qsize())
         else:
             self._shutdown_now = True
 
